package socialmedia;

import java.util.*;
import java.io.*;

/**
 * MiniSocialMedia is a minimally compiling, but non-functioning implementor
 * of the MiniSocialMediaPlatform interface.
 * 
 * @author 
 * @version 1.0
 */
public class MiniSocialMedia implements MiniSocialMediaPlatform {

	private HashMap<Integer, Account> accounts;
	
	public MiniSocialMedia()
	{
		this.accounts = new HashMap<>();
	}
	
	/**
	* The method creates an account in the platform with the given handle.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param handle account’s handle.
	* @throws IllegalHandleException if the handle already exists in the platform.
	* @throws InvalidHandleException if the new handle is empty, has more than 30
	* characters, or has white spaces.
	* @return the ID of the created account.
	*
	*/
	@Override
	public int createAccount(String handle) throws IllegalHandleException, InvalidHandleException {
		
		if(containsHandle(handle))
			throw new IllegalHandleException();
		
		if(handle == null || handle.length() <= 0 || handle.length() > 30)
			throw new InvalidHandleException();
		
		Account account = new Account();
		account.ID = Account.MAX_ID++;
		account.Handle = handle;
		account.Description = "";
		
		this.accounts.put(account.ID, account);
		
		return 0;
	}

	/**
	* The method removes the account with the corresponding ID from the platform.
	* When an account is removed, all of their posts and likes should also be
	* removed.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param id ID of the account.
	* @throws AccountIDNotRecognisedException if the ID does not match to any
	* account in the system.
	*/
	@Override
	public void removeAccount(int id) throws AccountIDNotRecognisedException {
		
		if(this.accounts.containsKey(id))
		{
			this.accounts.remove(id);
			return;
		}
            
		throw new AccountIDNotRecognisedException();
	}
	
	//Helper method
	private boolean containsHandle(String handle) {
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(handle))
			{
				return true;
			}
		}
		return false;
	}
	
	/**
	* The method replaces the oldHandle of an account by the newHandle.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param oldHandle account’s old handle.
	* @param newHandle account’s new handle.
	* @throws HandleNotRecognisedException if the old handle does not match to any
	* account in the system.
	* @throws IllegalHandleException if the new handle already exists in the
	* platform.
	* @throws InvalidHandleException if the new handle is empty, has more
	* than 30 characters, or has white spaces.
	*/
	@Override
	public void changeAccountHandle(String oldHandle, String newHandle)
			throws HandleNotRecognisedException, IllegalHandleException, InvalidHandleException {
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(oldHandle))
			{
				Account account = e.getValue();
				
				if(containsHandle(newHandle))
					throw new IllegalHandleException();
				
				if(newHandle == null || newHandle.length() <= 0 || newHandle.length() > 30)
					throw new InvalidHandleException();
				
				account.Handle = newHandle;
				return;
			}
		}

		throw new HandleNotRecognisedException();
	}

	/**
	* The method creates a formatted string summarising the stats of the account
	* identified by the given handle. The template should be:
	*
	* <pre>
	* ID: [account ID]
	* Handle: [account handle]
	* Description: [account description]
	* Post count: [total number of posts, including endorsements and replies]
	* Endorse count: [sum of endorsements received by each post of this account]
	* </pre>
	*
	* @param handle handle to identify the account.
	* @return the account formatted summary.
	* @throws HandleNotRecognisedException if the handle does not match to any
	* account in the system.
	*/
	@Override
	public String showAccount(String handle) throws HandleNotRecognisedException {
		// TODO Auto-generated method stub
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(handle))
			{				
				StringBuilder result = new StringBuilder();
		
				int postCount = e.getValue().posts.size(), 
					endorseCount = 0;
					
				result.append(String.format("ID: %d\n", e.getValue().ID));
				result.append(String.format("Handle: %s\n", e.getValue().Handle));
				result.append(String.format("Description: %s\n", e.getValue().Description));
				result.append(String.format("Post count: %d\n", postCount));
				result.append(String.format("Endorse count: %d", endorseCount));
				
				return result.toString();
			}
		}
		
		throw new HandleNotRecognisedException();
	}

	/**
	* The method creates a post for the account identified by the given handle with
	* the following message.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param handle handle to identify the account.
	* @param message post message.
	* @throws HandleNotRecognisedException if the handle does not match to any
	* account in the system.
	* @throws InvalidPostException if the message is empty or has more than
	* 100 characters.
	* @return the sequential ID of the created post.
	*/
	@Override
	public int createPost(String handle, String message) throws HandleNotRecognisedException, InvalidPostException {
		
		if(message == null || message.length() <= 0 || message.length() > 100)
			throw new InvalidPostException();
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(handle))
			{	
				return e.getValue().addPost(message);
			}
		}
		
		throw new HandleNotRecognisedException();
	}

	/**
	* The method creates an endorsement post of an existing post, similar to a
	* retweet on Twitter. An endorsement post is a special post. It contains a
	* reference to the endorsed post and its message is formatted as:
	* <p>
	* <code>"EP@" + [endorsed account handle] + ": " + [endorsed message]</code>
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param handle of the account endorsing a post.
	* @param id of the post being endorsed.
	* @return the sequential ID of the created post.
	* @throws HandleNotRecognisedException if the handle does not match to any
	* account in the system.
	* @throws PostIDNotRecognisedException if the ID does not match to any post in
	* the system.
	* @throws NotActionablePostException if the ID refers to a endorsement post.
	* Endorsement posts are not endorsable.
	* Endorsements are not transitive. For
	* instance, if post A is endorsed by post
	* B, and an account wants to endorse B, in
	* fact,the endorsement must refers to A.
	*/
	@Override
	public int endorsePost(String handle, int id)
			throws HandleNotRecognisedException, PostIDNotRecognisedException, NotActionablePostException {
		
		int endorsePostID = -1;
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(handle))
			{	
				endorsePostID = e.getValue().addEndorsementPost(handle, id);
				break;
			}
		}
		
		if(endorsePostID == -1)
			throw new HandleNotRecognisedException();
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().containsPost(id))
			{	
				if(!e.getValue().endorsePost(handle, id, endorsePostID))
					throw new NotActionablePostException();
				return endorsePostID;
			}
		}
		
		throw new PostIDNotRecognisedException();
	}

	/**
	* The method creates a comment post referring to an existing post, similarly to
	* a reply on Twitter. A comment post is a special post. It contains a reference
	* to the post being commented upon.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param handle of the account commenting a post.
	* @param id of the post being commented.
	* @param message the comment post message.
	* @return the sequential ID of the created post.
	* @throws HandleNotRecognisedException if the handle does not match to any
	* account in the system.
	* @throws PostIDNotRecognisedException if the ID does not match to any post in
	* the system.
	* @throws NotActionablePostException if the ID refers to a endorsement post.
	* Endorsement posts are not endorsable.
	* Endorsements cannot be commented. For
	* instance, if post A is endorsed by post
	* B, and an account wants to comment B, in
	* fact, the comment must refers to A.
	* @throws InvalidPostException if the comment message is empty or has
	* more than 100 characters.
	*/
	@Override
	public int commentPost(String handle, int id, String message) throws HandleNotRecognisedException,
			PostIDNotRecognisedException, NotActionablePostException, InvalidPostException {
				
		int commentPostID = -1;
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().Handle.equals(handle))
			{	
				commentPostID = e.getValue().addCommentPost(handle, id, message);
				break;
			}
		}
		
		if(commentPostID == -1)
			throw new HandleNotRecognisedException();
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().containsPost(id))
			{	
				if(!e.getValue().commentPost(handle, id, commentPostID))
					throw new NotActionablePostException();
				return commentPostID;
			}
		}
		
		throw new PostIDNotRecognisedException();
	}

	/**
	* The method removes the post from the platform. When a post is removed, all
	* its endorsements should be removed as well. All replies to this post should
	* be updated by replacing the reference to this post by a generic empty post.
	* <p>
	* The generic empty post message should be "The original content was removed
	* from the system and is no longer available.". This empty post is just a
	* replacement placeholder for the post which a reply refers to. Empty posts
	* should not be linked to any account and cannot be acted upon, i.e., it cannot
	* be available for endorsements or replies.
	* <p>
	* The state of this SocialMediaPlatform must be be unchanged if any exceptions
	* are thrown.
	*
	* @param id ID of post to be removed.
	* @throws PostIDNotRecognisedException if the ID does not match to any post in
	* the system.
	*/
	@Override
	public void deletePost(int id) throws PostIDNotRecognisedException {
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().deletePost(id))
			{	
				return;
			}
		}
		
		throw new PostIDNotRecognisedException();
	}

	/**
	* The method generates a formated string containing the details of a single
	* post. The format is as follows:
	*
	* @param id of the post to be shown.
	* @return a formatted string containing post’s details.
	* @throws PostIDNotRecognisedException if the ID does not match to any post in
	* the system.
	*/
	@Override
	public String showIndividualPost(int id) throws PostIDNotRecognisedException {
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().containsPost(id))
			{				
				StringBuilder result = new StringBuilder();
		
				result.append(e.getValue().showIndividualPost(id, 0));
				
				return result.toString();
			}
		}
		
		throw new PostIDNotRecognisedException();
		
	}
	
	//Helper method
	private StringBuilder showPostChildrenDetails(StringBuilder sb, HashMap<Integer, Post> overAllposts, int id, int level) {
		
		if(!(overAllposts.get(id) instanceof EndorsementPost))
		{
			String res = overAllposts.get(id).showIndividualPost(id, level);
			sb.append(res);
			
			int commensCount = overAllposts.get(id).comments.size();
			
			if(commensCount > 0)
				sb.append("|\n");
			
			for(int i = 0; i < commensCount; i++) {
				
				int commentPostID = overAllposts.get(id).comments.get(i);
				
				StringBuilder commentResult = showPostChildrenDetails(new StringBuilder(), overAllposts, commentPostID, level + 1);
				sb.append(commentResult.toString());
			}
		}
		return sb;
	}
	
	/**
	* The method builds a StringBuilder showing the details of the current post and
	* all its children posts. The format is as follows (you can use tabs or spaces to represent
	indentation):
	*
	* @param id of the post to be shown.
	* @return a formatted StringBuilder containing the details of the post and its
	* children.
	* @throws PostIDNotRecognisedException if the ID does not match to any post in
	* the system.
	* @throws NotActionablePostException if the ID refers to an endorsement post.
	* Endorsement posts do not have children
	* since they are not endorsable nor
	* commented.
	*/
	@Override
	public StringBuilder showPostChildrenDetails(int id)
			throws PostIDNotRecognisedException, NotActionablePostException {
		
		HashMap<Integer, Post> overAllposts = new HashMap<>();
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			for (Map.Entry<Integer, Post> p : e.getValue().posts.entrySet()) {
				Post post = p.getValue();
				overAllposts.put(post.ID, post);
			}
		}
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			if(e.getValue().containsPost(id))
			{			
				if(e.getValue().isEndorsementPostPost(id))
					throw new NotActionablePostException();
				
				StringBuilder sb = showPostChildrenDetails(new StringBuilder(), overAllposts, id, 0);
				return sb;
			}
		}
		
		throw new PostIDNotRecognisedException();
	}

	/**
	* This method identifies and returns the post with the most number of
	* endorsements, a.k.a. the most popular post.
	*
	* @return the ID of the most popular post.
	*/
	@Override
	public int getMostEndorsedPost() {
		int maxCount = 0, mostEndorsedPost = -1;
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			for (Map.Entry<Integer, Post> p : e.getValue().posts.entrySet())
			{
				Post post = p.getValue();
				int endorsementsCount = post.endorsements.size();
				if(endorsementsCount > maxCount) {
					maxCount = endorsementsCount;
					mostEndorsedPost = post.ID;
				}
			}
		}
		
		return mostEndorsedPost;
	}

	/**
	* This method identifies and returns the account with the most number of
	* endorsements, a.k.a. the most popular account.
	*
	* @return the ID of the most popular account.
	*/
	@Override
	public int getMostEndorsedAccount() {
		int maxCount = 0, mostEndorsedAccount = -1;
		
		for (Map.Entry<Integer, Account> e : this.accounts.entrySet())
		{
			int totalEndorsementsCount = 0;
			for (Map.Entry<Integer, Post> p : e.getValue().posts.entrySet()) {
				Post post = p.getValue();
				totalEndorsementsCount += post.endorsements.size();
			}
			
			if(totalEndorsementsCount > maxCount) {
				maxCount = totalEndorsementsCount;
				mostEndorsedAccount = e.getValue().ID;
			}
		}
		
		return mostEndorsedAccount;
	}

	/**
	* Method empties this SocialMediaPlatform of its contents and resets all
	* internal counters.
	*/
	@Override
	public void erasePlatform() {
		this.accounts.clear();
	}

	/**
	* Method saves this SocialMediaPlatform’s contents into a serialised file, with
	* the filename given in the argument.
	*
	* @param filename location of the file to be saved
	* @throws IOException if there is a problem experienced when trying to save the
	* store contents to the file
	*/
	@Override
	public void savePlatform(String filename) throws IOException {
		
		try
        {  
            FileOutputStream file = new FileOutputStream(filename);
            ObjectOutputStream out = new ObjectOutputStream(file);
             
            out.writeObject(this.accounts);
             
            out.close();
            file.close(); 
        }
        catch(IOException ex)
        {
            throw new IOException();
        }
	}

	/**
	* Method should load and replace this SocialMediaPlatform’s contents with the
	* serialised contents stored in the file given in the argument.
	* <p>
	* The state of this SocialMediaPlatform’s must be be unchanged if any
	* exceptions are thrown.
	*
	* @param filename location of the file to be loaded
	* @throws IOException if there is a problem experienced when trying
	* to load the store contents from the file
	* @throws ClassNotFoundException if required class files cannot be found when
	* loading
	*/
	@Override
	public void loadPlatform(String filename) throws IOException, ClassNotFoundException {
		try
        {  
            FileInputStream file = new FileInputStream(filename);
            ObjectInputStream in = new ObjectInputStream(file);
             
            this.accounts = (HashMap<Integer, Account>)in.readObject();
             
            in.close();
            file.close();
        }
        catch(IOException ex)
        {
            throw new IOException();
        }
        catch(ClassNotFoundException ex)
        {
            throw new ClassNotFoundException();
        }
	}

}
